==> puppetlabs-comply-2.2.0.tar.gz: puppetlabs-comply-2.2.0/lib/task_helper.rb <==
# frozen_string_literal: true

require 'json'
require 'open3'

def get_fact(fact)
  if Gem.win_platform?
    require 'win32/registry'
    installed_dir =
      begin
        Win32::Registry::HKEY_LOCAL_MACHINE.open('SOFTWARE\Puppet Labs\Puppet') do |reg|
          # rubocop:disable Style/RescueModifier
          # Rescue missing key
          dir = reg['RememberedInstallDir64'] rescue ''
          # Both keys may exist, make sure the dir exists
          break dir if File.exist?(dir)
          # Rescue missing key
          reg['RememberedInstallDir'] rescue ''
          # rubocop:enable Style/RescueModifier
        end
      rescue Win32::Registry::Error
        # Rescue missing registry path
        ''
      end
    facter =
      if installed_dir.empty?
        ''
      else
        File.join(installed_dir, 'bin', 'facter.bat')
      end

  else
    facter = '/opt/puppetlabs/puppet/bin/facter'
  end

  # Fall back to PATH lookup if puppet-agent isn't installed
  facter = 'facter' unless File.exist?(facter)
  cmd = [facter, '--json']
  cmd << fact if fact
  stdout, stderr, status = Open3.capture3(*cmd)
  raise "Exit #{status.exitstatus} running #{cmd.join(' ')}: #{stderr}" unless status.success?
  JSON.parse(stdout)
end

# accepts an array of strings, eg ['config', 'print']
def run_puppet(sub_command)
  if Gem.win_platform?
    require 'win32/registry'
    installed_dir =
      begin
        Win32::Registry::HKEY_LOCAL_MACHINE.open('SOFTWARE\Puppet Labs\Puppet') do |reg|
          # rubocop:disable Style/RescueModifier
          # Rescue missing key
          dir = reg['RememberedInstallDir64'] rescue ''
          # Both keys may exist, make sure the dir exists
          break dir if File.exist?(dir)
          # Rescue missing key
          reg['RememberedInstallDir'] rescue ''
          # rubocop:enable Style/RescueModifier
        end
      rescue Win32::Registry::Error
        # Rescue missing registry path
        ''
      end
    puppet =
      if installed_dir.empty?
        ''
      else
        File.join(installed_dir, 'bin', 'puppet.bat')
      end

  else
    puppet = '/opt/puppetlabs/puppet/bin/puppet'
  end

  # Fall back to PATH lookup if puppet-agent isn't installed
  puppet = 'puppet' unless File.exist?(puppet)
  cmd = [puppet] + sub_command
  stdout, stderr, status = Open3.capture3(*cmd)
  raise "Exit #{status.exitstatus} running #{cmd.join(' ')}: #{stderr}" unless status.success?
  stdout.strip
end

def return_install_directories
  os_facts = get_fact('os')
  case os_facts['os']['family']
  when 'windows'
    install_path = 'C:/ProgramData/PuppetLabs/comply/Assessor-CLI/'
    result_path = 'C:/ProgramData/PuppetLabs/comply/tmp/'
  else
    install_path = '/opt/puppetlabs/comply/Assessor-CLI/'
    result_path = '/opt/puppetlabs/comply/tmp/'
  end
  [install_path, result_path]
end

def upload_scan_result(result_file: nil, format: nil, comply_server: nil, comply_port: nil, custom_profile_id: nil, scan_type: 'adhoc', certname: nil, ssl_verify_mode: 'peer', scan_id: nil)
  os_facts = get_fact('os')
  # parse report for facts
  path = case os_facts['os']['family']
         when 'windows'
           ''
         else
           # fix for missing path in RHEL6 vm
           'export PATH=/usr/local/bin:$PATH && '
         end

  path_to_result_file = if result_file.nil?
                          File.join(Dir.tmpdir, 'xccdf-results.xml')
                        else
                          result_file
                        end
  request = Net::HTTP::Post.new('/q/api/v3/core/store/report')
  form_data = [['report', File.open(path_to_result_file)], ['scanType', scan_type], ['format', format], ['certname', certname]]
  form_data.push(['customProfileID', custom_profile_id.to_s]) unless custom_profile_id.nil?
  form_data.push(['scanId', scan_id]) unless scan_id.nil?
  request.set_form form_data, 'multipart/form-data'
  # send the report to the scarp server
  config = Open3.capture3("#{path}puppet config print ssldir")
  ssldir = config[0].strip

  verify_mode = case ssl_verify_mode
                when 'peer' then OpenSSL::SSL::VERIFY_PEER
                when 'none' then OpenSSL::SSL::VERIFY_NONE
                else raise "Invalid SSL verify mode '#{ssl_verify_mode}. Either 'peer' or 'none' should be used.'"
                end

  response = Net::HTTP.start(
    comply_server,
    comply_port.to_i,
    use_ssl: true,
    key: OpenSSL::PKey::RSA.new(File.read("#{ssldir}/private_keys/#{certname}.pem")),
    cert: OpenSSL::X509::Certificate.new(File.read("#{ssldir}/certs/#{certname}.pem")),
    ca_file: "#{ssldir}/certs/ca.pem",
    verify_mode: verify_mode,
    verify_depth: 5,
  ) do |http|
    http.request(request)
  end
  response
end

==> puppetlabs-comply-3.0.0.tar.gz: puppetlabs-comply-3.0.0/lib/task_helper.rb <==
# frozen_string_literal: true

require 'json'
require 'open3'
require 'zlib'

def get_fact(fact)
  if Gem.win_platform?
    require 'win32/registry'
    installed_dir =
      begin
        Win32::Registry::HKEY_LOCAL_MACHINE.open('SOFTWARE\Puppet Labs\Puppet') do |reg|
          # rubocop:disable Style/RescueModifier
          # Rescue missing key
          dir = reg['RememberedInstallDir64'] rescue ''
          # Both keys may exist, make sure the dir exists
          break dir if File.exist?(dir)
          # Rescue missing key
          reg['RememberedInstallDir'] rescue ''
          # rubocop:enable Style/RescueModifier
        end
      rescue Win32::Registry::Error
        # Rescue missing registry path
        ''
      end
    facter =
      if installed_dir.empty?
        ''
      else
        File.join(installed_dir, 'bin', 'facter.bat')
      end

  else
    facter = '/opt/puppetlabs/puppet/bin/facter'
  end

  # Fall back to PATH lookup if puppet-agent isn't installed
  facter = 'facter' unless File.exist?(facter)
  cmd = [facter, '--json']
  cmd << fact if fact
  stdout, stderr, status = Open3.capture3(*cmd)
  raise "Exit #{status.exitstatus} running #{cmd.join(' ')}: #{stderr}" unless status.success?
  JSON.parse(stdout)
end

# accepts an array of strings, eg ['config', 'print']
def run_puppet(sub_command)
  if Gem.win_platform?
    require 'win32/registry'
    installed_dir =
      begin
        Win32::Registry::HKEY_LOCAL_MACHINE.open('SOFTWARE\Puppet Labs\Puppet') do |reg|
          # rubocop:disable Style/RescueModifier
          # Rescue missing key
          dir = reg['RememberedInstallDir64'] rescue ''
          # Both keys may exist, make sure the dir exists
          break dir if File.exist?(dir)
          # Rescue missing key
          reg['RememberedInstallDir'] rescue ''
          # rubocop:enable Style/RescueModifier
        end
      rescue Win32::Registry::Error
        # Rescue missing registry path
        ''
      end
    puppet =
      if installed_dir.empty?
        ''
      else
        File.join(installed_dir, 'bin', 'puppet.bat')
      end

  else
    puppet = '/opt/puppetlabs/puppet/bin/puppet'
  end

  # Fall back to PATH lookup if puppet-agent isn't installed
  puppet = 'puppet' unless File.exist?(puppet)
  cmd = [puppet] + sub_command
  stdout, stderr, status = Open3.capture3(*cmd)
  raise "Exit #{status.exitstatus} running #{cmd.join(' ')}: #{stderr}" unless status.success?
  stdout = stdout[0] if stdout.is_a?(Array)
  stdout.strip
end

def return_install_directories
  os_facts = get_fact('os')
  case os_facts['os']['family']
  when 'windows'
    install_path = 'C:/ProgramData/PuppetLabs/comply/Assessor-CLI/'
    result_path = 'C:/ProgramData/PuppetLabs/comply/tmp/'
  else
    install_path = '/opt/puppetlabs/comply/Assessor-CLI/'
    result_path = '/opt/puppetlabs/comply/tmp/'
  end
  [install_path, result_path]
end

def get_result_content(result_file: nil, compression_mode: 'none')
  path_to_result_file = if result_file.nil?
                          File.join(Dir.tmpdir, 'xccdf-results.xml')
                        else
                          result_file
                        end

  uncompressed = File.open(path_to_result_file)

  if compression_mode == 'gzip'
    StringIO.new Zlib.gzip(uncompressed.to_s)
  else
    uncompressed
  end
end

def upload_scan_result(result_file: nil, format: nil, comply_server: nil, comply_port: nil, custom_profile_id: nil,
                       scan_type: 'ad hoc', certname: nil, ssl_verify_mode: 'peer', scan_id: nil, auth_token: nil,
                       scan_report_upload_path: '/api/v3/core/store/report', compression_mode: nil)

  request = Net::HTTP::Post.new(scan_report_upload_path)
  request['Authorization'] = auth_token

  report = get_result_content(result_file: result_file, compression_mode: compression_mode)

  form_data = [['report', report], ['scanType', scan_type], ['format', format], ['certname', certname]]
  form_data.push(['customProfileID', custom_profile_id.to_s]) unless custom_profile_id.nil?
  form_data.push(['scanId', scan_id]) unless scan_id.nil?
  request.set_form form_data, 'multipart/form-data'
  # send the report to the scarp server
  ssldir = run_puppet(['config', 'print', 'ssldir'])

  verify_mode = case ssl_verify_mode
                when 'peer' then OpenSSL::SSL::VERIFY_PEER
                when 'none' then OpenSSL::SSL::VERIFY_NONE
                else raise "Invalid SSL verify mode '#{ssl_verify_mode}. Either 'peer' or 'none' should be used.'"
                end

  opt_hash = { use_ssl: true, ca_file: "#{ssldir}/certs/ca.pem", verify_mode: verify_mode, verify_depth: 5 }

  # only specify files for agent certs in 'peer' mode (i.e., using Comply On-prem)
  if verify_mode == OpenSSL::SSL::VERIFY_PEER
    opt_hash[:cert] = OpenSSL::X509::Certificate.new(File.read("#{ssldir}/certs/#{certname}.pem"))
    opt_hash[:key] = OpenSSL::PKey::RSA.new(File.read("#{ssldir}/private_keys/#{certname}.pem"))
  end

  response = Net::HTTP.start(
    comply_server,
    comply_port.to_i,
    opt_hash,
  ) do |http|
    http.request(request)
  end
  response
end
